# Copyright (C) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See LICENSE in project root for information.


import sys
if sys.version >= '3':
    basestring = str

from pyspark.ml.param.shared import *
from pyspark import keyword_only
from pyspark.ml.util import JavaMLReadable, JavaMLWritable
from pyspark.ml.wrapper import JavaTransformer, JavaEstimator, JavaModel
from pyspark.ml.common import inherit_doc
from mmlspark.Utils import *

@inherit_doc
class _CNTKModel(ComplexParamsMixin, JavaMLReadable, JavaMLWritable, JavaTransformer):
    """
    The ``CNTKModel`` evaluates a pre-trained CNTK model in parallel.  The
    ``CNTKModel`` takes a path to a model and automatically loads and
    distributes the model to workers for parallel evaluation using CNTK's
    java bindings.

    The ``CNTKModel`` loads the pretrained model into the ``Function`` class
    of CNTK.  One can decide which node of the CNTK Function computation
    graph to evaluate by passing in the name of the output node with the
    output node parameter.  Currently the ``CNTKModel`` supports single
    input single output models.

    The ``CNTKModel`` takes an input column which should be a column of
    spark vectors and returns a column of spark vectors representing the
    activations of the selected node.  By default, the CNTK model defaults
    to using the model's first input and first output node.

    Args:

        inputCol (str): The name of the input column
        inputNode (int): Index of the input node (default: 0)
        miniBatchSize (int): Size of minibatches (default: 10)
        model (object): Array of bytes containing the serialized CNTKModel
        outputCol (str): The name of the output column
        outputNodeIndex (int): Index of the output node
        outputNodeName (str): Name of the output node
    """

    @keyword_only
    def __init__(self, inputCol=None, inputNode=0, miniBatchSize=10, model=None, outputCol=None, outputNodeIndex=None, outputNodeName=None):
        super(_CNTKModel, self).__init__()
        self._java_obj = self._new_java_obj("com.microsoft.ml.spark.CNTKModel")
        self.inputCol = Param(self, "inputCol", "inputCol: The name of the input column")
        self.inputNode = Param(self, "inputNode", "inputNode: Index of the input node (default: 0)")
        self._setDefault(inputNode=0)
        self.miniBatchSize = Param(self, "miniBatchSize", "miniBatchSize: Size of minibatches (default: 10)")
        self._setDefault(miniBatchSize=10)
        self.model = Param(self, "model", "model: Array of bytes containing the serialized CNTKModel")
        self.outputCol = Param(self, "outputCol", "outputCol: The name of the output column")
        self.outputNodeIndex = Param(self, "outputNodeIndex", "outputNodeIndex: Index of the output node")
        self.outputNodeName = Param(self, "outputNodeName", "outputNodeName: Name of the output node")
        if hasattr(self, "_input_kwargs"):
            kwargs = self._input_kwargs
        else:
            kwargs = self.__init__._input_kwargs
        self.setParams(**kwargs)

    @keyword_only
    def setParams(self, inputCol=None, inputNode=0, miniBatchSize=10, model=None, outputCol=None, outputNodeIndex=None, outputNodeName=None):
        """
        Set the (keyword only) parameters

        Args:

            inputCol (str): The name of the input column
            inputNode (int): Index of the input node (default: 0)
            miniBatchSize (int): Size of minibatches (default: 10)
            model (object): Array of bytes containing the serialized CNTKModel
            outputCol (str): The name of the output column
            outputNodeIndex (int): Index of the output node
            outputNodeName (str): Name of the output node
        """
        if hasattr(self, "_input_kwargs"):
            kwargs = self._input_kwargs
        else:
            kwargs = self.__init__._input_kwargs
        return self._set(**kwargs)

    def setInputCol(self, value):
        """

        Args:

            inputCol (str): The name of the input column

        """
        self._set(inputCol=value)
        return self


    def getInputCol(self):
        """

        Returns:

            str: The name of the input column
        """
        return self.getOrDefault(self.inputCol)


    def setInputNode(self, value):
        """

        Args:

            inputNode (int): Index of the input node (default: 0)

        """
        self._set(inputNode=value)
        return self


    def getInputNode(self):
        """

        Returns:

            int: Index of the input node (default: 0)
        """
        return self.getOrDefault(self.inputNode)


    def setMiniBatchSize(self, value):
        """

        Args:

            miniBatchSize (int): Size of minibatches (default: 10)

        """
        self._set(miniBatchSize=value)
        return self


    def getMiniBatchSize(self):
        """

        Returns:

            int: Size of minibatches (default: 10)
        """
        return self.getOrDefault(self.miniBatchSize)


    def setModel(self, value):
        """

        Args:

            model (object): Array of bytes containing the serialized CNTKModel

        """
        self._set(model=value)
        return self


    def getModel(self):
        """

        Returns:

            object: Array of bytes containing the serialized CNTKModel
        """
        return self.getOrDefault(self.model)


    def setOutputCol(self, value):
        """

        Args:

            outputCol (str): The name of the output column

        """
        self._set(outputCol=value)
        return self


    def getOutputCol(self):
        """

        Returns:

            str: The name of the output column
        """
        return self.getOrDefault(self.outputCol)


    def setOutputNodeIndex(self, value):
        """

        Args:

            outputNodeIndex (int): Index of the output node

        """
        self._set(outputNodeIndex=value)
        return self


    def getOutputNodeIndex(self):
        """

        Returns:

            int: Index of the output node
        """
        return self.getOrDefault(self.outputNodeIndex)


    def setOutputNodeName(self, value):
        """

        Args:

            outputNodeName (str): Name of the output node

        """
        self._set(outputNodeName=value)
        return self


    def getOutputNodeName(self):
        """

        Returns:

            str: Name of the output node
        """
        return self.getOrDefault(self.outputNodeName)



    @classmethod
    def read(cls):
        """ Returns an MLReader instance for this class. """
        return JavaMMLReader(cls)

    @staticmethod
    def getJavaPackage():
        """ Returns package name String. """
        return "com.microsoft.ml.spark.CNTKModel"

    @staticmethod
    def _from_java(java_stage):
        module_name=_CNTKModel.__module__
        module_name=module_name.rsplit(".", 1)[0] + ".CNTKModel"
        return from_java(java_stage, module_name)
